from channels.generic.websocket import JsonWebsocketConsumer


class KnockerConsumer(JsonWebsocketConsumer):
    def websocket_connect(self, message):
        """
        Attach the consumer to the selected language
        """
        self.groups = self.get_groups()
        return super().websocket_connect(message)

    def get_groups(self):
        """
        Attach the consumer to the selected language
        """
        lang = self.scope["url_route"]["kwargs"].get("language")
        return [
            "knocker-%s" % lang,
        ]

    def knocker_saved(self, event):
        """
        This method handles messages sent to knocker.saved

        :param event: event object
        """
        self.send_json(content=event["message"])



from channels.routing import URLRouter
from django.urls import path

from .consumers import KnockerConsumer

# consumers can be freely appended: path ensure the correct match
channel_routing = URLRouter(
    [
        path("notification/<str:language>/", KnockerConsumer.as_asgi()),
    ]
)



import json

import pytest
from channels.db import database_sync_to_async
from channels.testing import WebsocketCommunicator
from django.utils.text import slugify

from tests.example_app.models import NoKnockPost, Post
from tests.example_app.routing import application


async def _connect():
    path = "knocker/notification/en/"
    communicator = WebsocketCommunicator(application, path)
    connected, __ = await communicator.connect()
    assert connected
    assert await communicator.receive_nothing() is True
    return communicator


@pytest.mark.asyncio
@pytest.mark.django_db
async def test_consumer_notification():
    @database_sync_to_async
    def get_post(title):
        return Post.objects.create(
            title=title,
            slug=slugify(title),
        )

    @database_sync_to_async
    def get_noknock_post(title):
        return NoKnockPost.objects.create(
            title=title,
            slug=slugify(title),
        )

    communicator = await _connect()

    # Post type without notifications
    await get_noknock_post("first post")
    assert await communicator.receive_nothing() is True

    # Post type which return notifications
    post = await get_post("first post")
    data = await communicator.receive_json_from()
    notification = json.loads(data)
    assert notification["title"] == "new {}".format(post._meta.verbose_name)
    assert notification["message"] == post.title
    assert notification["url"] == post.get_absolute_url()

    await communicator.disconnect()



from unittest.mock import patch

from django.db.models.signals import post_save
from django.utils.translation import get_language, override
from parler.utils.context import smart_override

from knocker.signals import active_knocks, pause_knocks

from .base import BaseKnocker
from .example_app.models import MultiLanguagePost, NoKnockPost, Post


class KnockerTest(BaseKnocker):
    def tearDown(self):
        MultiLanguagePost._disconnect()
        Post._disconnect()
        super().tearDown()

    def test_model_attributes(self):
        posts = []
        posts.append(
            MultiLanguagePost.objects.create(
                title="first post",
                slug="first-post",
            )
        )
        posts.append(
            MultiLanguagePost.objects.create(
                title="second post",
                slug="second-post",
            )
        )

        for language in [get_language()]:
            with override(language):
                for post in posts:
                    knock_create = post.as_knock("post_save", True)
                    self.assertEqual(knock_create["title"], "new {}".format(post._meta.verbose_name))
                    self.assertEqual(knock_create["message"], post.title)
                    self.assertEqual(knock_create["language"], language)

    def test_parler_model_attributes(self):
        posts = []
        with smart_override("en"):
            posts.append(
                MultiLanguagePost.objects.create(
                    title="first post",
                    slug="first-post",
                )
            )
            posts.append(
                MultiLanguagePost.objects.create(
                    title="second post",
                    slug="second-post",
                )
            )
        posts[0].create_translation("it", title="primo articolo", slug="primo-articolo")
        posts[1].create_translation("it", title="secondo articolo", slug="secondo-articolo")
        posts[0].create_translation("fr", title="premier article", slug="premier-article")

        for post in posts:
            for language in post.get_available_languages():
                with smart_override(language):
                    post.set_current_language(language)
                    if language != "fr":
                        knock_create = post.as_knock("post_save", True)
                        self.assertEqual(
                            knock_create["title"],
                            "new {}".format(post._meta.verbose_name),
                        )
                        self.assertEqual(knock_create["message"], post.title)
                        self.assertEqual(knock_create["language"], language)
                        post.send_knock("post_save", True)

                        knock_create = post.as_knock("post_save", False)
                        self.assertEqual(
                            knock_create["title"],
                            "new {}".format(post._meta.verbose_name),
                        )
                        self.assertEqual(knock_create["message"], post.title)
                        self.assertEqual(knock_create["language"], language)
                        post.send_knock("post_save", False)
                    else:
                        self.assertFalse(post.should_knock("post_save"))
                        self.assertFalse(post.as_knock("post_save", True))
                        self.assertFalse(post.as_knock("post_save", False))
                        post.send_knock("post_save", False)

    @patch("knocker.signals.notify_items_post_save", autospec=True)
    def test_no_knock(self, handler):
        posts = []
        post_save.connect(handler, NoKnockPost, dispatch_uid="test_knocker_mock")
        posts.append(
            NoKnockPost.objects.create(
                title="first post",
                slug="first-post",
            )
        )
        posts.append(
            NoKnockPost.objects.create(
                title="second post",
                slug="second-post",
            )
        )
        self.assertEqual(handler.call_count, 2)

    @patch("knocker.mixins.notify_items_post_save")
    def test_signal(self, handler):
        post = Post.objects.create(
            title="signal post",
            slug="signal-post",
        )
        handler.assert_called_once()
        self.assertTrue(handler.return_value)
        handler.reset_mock()

        post.title = "mod title"
        post.save()
        handler.assert_called_once()
        self.assertTrue(handler.return_value)
        handler.reset_mock()

        with pause_knocks(post):
            post.title = "pause title"
            self.assertFalse(active_knocks(post))
            post.save()
            handler.assert_not_called()

        post.title = "mod title"
        post.save()
        handler.assert_called_once()
        self.assertTrue(handler.return_value)
        handler.reset_mock()
